'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = apAuthenticate;
/*
 * Promise-based wrapper around passport.authenticate (allows for more
 * flexible usage and negates dependency on express)
 */

function apAuthenticate(strategy, query) {
  var self = this;

  return new Promise(function (resolve, reject) {

    var authenticator = self._authenticators[strategy];
    if (!authenticator) return reject(new Error('no authenticator for strategy: ' + strategy));

    var logInCalled = false;
    var fakeReq = {
      query: query,
      logIn: function logIn(user, options, callback) {
        logInCalled = { user: user, options: options };
        callback();
      }
    };

    var fakeRes = {
      headers: {}
    };

    var fakeNext = function fakeNext() {
      // Must get Apollo style errors/data property.
      var data = {
        type: 'loginComplete',
        key: 'oauth2'
      };

      // XXX can we get an error TO here?  and, errrors CURRENTLY NOT CAUGHT HERE
      if (0 /* server side error */) {
          /* istanbul ignore next */
          data.errors = []; // apollo style error TODO
        } else {
        if (logInCalled) data.data = { oauth2: {
            token: self.createTokenFromUser(logInCalled.user),
            error: ""
          } };else data.data = { oauth2: {
            token: "",
            error: "Authentication Failed"
          } };
      }

      resolve(data);
    };

    authenticator(fakeReq, fakeRes, fakeNext);
  });
}