'use strict';

var _chai = require('chai');

var _chai2 = _interopRequireDefault(_chai);

var _chaiAsPromised = require('chai-as-promised');

var _chaiAsPromised2 = _interopRequireDefault(_chaiAsPromised);

var _authenticate = require('./authenticate');

var _authenticate2 = _interopRequireDefault(_authenticate);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var should = _chai2.default.should();
_chai2.default.use(_chaiAsPromised2.default);

describe('apollo-passport - apAuthenticate', function () {

  var context = {
    createTokenFromUser: function createTokenFromUser() {
      return 'token';
    },
    _authenticators: {
      FakePassing: function FakePassing(req, res, next) {
        req.logIn({ id: 1 }, {}, next);
      },
      FakeFailing: function FakeFailing(req, res, next) {
        next();
      }
    }
  };

  it('rejects on missing strategy', function () {
    return _authenticate2.default.call(context, 'missing').should.reject;
  });

  it('resolves with a token on success', function () {
    return _authenticate2.default.call(context, 'FakePassing').should.become({
      type: 'loginComplete',
      key: 'oauth2',
      data: {
        oauth2: {
          token: 'token',
          error: ''
        }
      }
    });
  });

  it('resolves with an error on auth failure', function () {
    return _authenticate2.default.call(context, 'FakeFailing').should.become({
      type: 'loginComplete',
      key: 'oauth2',
      data: {
        oauth2: {
          token: '',
          error: 'Authentication Failed'
        }
      }
    });
  });
});