'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.defaultCreateTokenFromUser = exports.defaultMapUserToJWTProps = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _path2 = require('path');

var _path3 = _interopRequireDefault(_path2);

var _passport = require('passport');

var _passport2 = _interopRequireDefault(_passport);

var _jsonwebtoken = require('jsonwebtoken');

var _jsonwebtoken2 = _interopRequireDefault(_jsonwebtoken);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _authenticate = require('./authenticate');

var _authenticate2 = _interopRequireDefault(_authenticate);

var _graphqlMerge = require('./utils/graphql-merge');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

// Some super weird babel thing going on here, where path is not defined.
var _path = _path3.default;

// Non-standard, see
// http://www.iana.org/assignments/jwt/jwt.xhtml
var defaultMapUserToJWTProps = function defaultMapUserToJWTProps(user) {
  return { userId: user.id };
};

function defaultCreateTokenFromUser(user) {
  return _jsonwebtoken2.default.sign(this.mapUserToJWTProps(user), this.jwtSecret, { expiresIn: "7 days" });
}

var ApolloPassport = function () {
  function ApolloPassport() {
    var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    _classCallCheck(this, ApolloPassport);

    // TODO actually let's rather default to a DB request on each query if no jwt secret specified
    var jwtSecret = options.jwtSecret || process.env.JWT_SECRET;
    if (!jwtSecret && !options.createTokenFromUser) throw new Error("no jwtSecret nor a custom createTokenFromUser specified");

    this._useJwt = true;

    this.strategies = {};
    this._winston = options.winston || require('winston');
    this._resolvers = require('./resolvers').default;
    this._schema = require('./schema').default;
    this._authenticators = {};
    this.passport = options.passport || _passport2.default;

    this.apAuthenticate = _authenticate2.default;

    this.ROOT_URL = options.ROOT_URL || process.env.ROOT_URL || typeof ROOT_URL === 'string' && ROOT_URL;
    if (!this.ROOT_URL) throw new Error("No ROOT_URL set.  Please see Apollo Passport README");
    if (!this.ROOT_URL.endsWith('/')) this.ROOT_URL += '/';

    this.authPath = options.authPath || 'ap-auth';
    if (this.authPath.startsWith('/')) this.authPath = this.authPath.substr(1);

    this.authUrlRoot = this.ROOT_URL + this.authPath;

    this.assertIsDBDriver(options.db);
    this.db = options.db;

    this.jwtSecret = jwtSecret;
    this.mapUserToJWTProps = options.mapUserToJWTProps || defaultMapUserToJWTProps;
    this.createTokenFromUser = options.createTokenFromUser || defaultCreateTokenFromUser;
  }

  _createClass(ApolloPassport, [{
    key: 'assertIsDBDriver',
    value: function assertIsDBDriver(db) {
      if (!db) throw new Error("Must provide a { db: DBDriverInstance } option");
      if (!(db.createUser && db.fetchUserByEmail)) throw new Error("Option 'db' must be a valid DBDriver instance, e.g. " + "new RethinkDBDriverDash(r)");
    }
  }, {
    key: 'use',
    value: function use(nameArg, Strategy, options, verify) {
      var parts = nameArg.split(':', 2);
      var name = parts.pop(),
          namespace = parts.pop();

      if (!verify && typeof options === 'function') {
        verify = options;
        options = null;
      }

      var isAugmented = Strategy.__isAugmented;

      /*
       * Now that AugmentedStrategies include their own deps, let's aim to replace
       * this with an apolloPassport.addClass('oauth2') or something.
       */
      if (!options) options = this.require(name, 'defaultOptions', namespace, isAugmented /* optional */);
      if (!verify) verify = this.require(name, 'verify', namespace, isAugmented /* optional */);

      if (namespace === 'oauth' || namespace === 'oauth2') {
        if (!options.callbackURL) options.callbackURL = this.authUrlRoot + '/' + name + '/callback';
        // if (!options.profileFields)
        //  options.profileFields = // Passport converts these per strategy
        //    [ 'id', 'username', 'displayName', 'gender', 'emails', 'photos' ];
      }

      // console.log(options);

      if (isAugmented) {
        var _instance = new Strategy(this, options);
        this.passport.use(_instance.strategy);
        this.strategies[name] = _instance; // check up on this, conflit with non-augmented
        this._authenticators[name] = _passport2.default.authenticate(name); // options, etc. from below

        if (_instance.resolvers) this._resolvers = (0, _graphqlMerge.mergeResolvers)(this._resolvers, _instance.resolvers);
        if (_instance.schema) this._schema = [(0, _graphqlMerge.mergeSchemas)([this._schema[0], _instance.schema[0]])];

        return;
      }

      var instance = new Strategy(options, namespace ? verify.bind(this, name) : verify.bind(this));
      this.passport.use(instance);

      // this._strategies.set(instance.name, instance);
      this.strategies[name] = instance;

      // TODO, make local use this too
      this._authenticators[name] = _passport2.default.authenticate(name);

      // Not used yet except for options (TODO serve on auth URL)
      if (options.scope) this.passport.authenticate(name, { scope: options.scope });

      var apWrapper = this.require(name, 'index', namespace, true /* optional */);
      if (apWrapper) this.strategies[name] = new apWrapper(this);

      var resolvers = this.require(name, 'resolvers', namespace, true /* optional */);
      if (resolvers) this._resolvers = (0, _graphqlMerge.mergeResolvers)(this._resolvers, resolvers);

      var schema = this.require(name, 'schema', namespace, true /* optional */);
      if (schema) this._schema = [(0, _graphqlMerge.mergeSchemas)([this._schema[0], schema[0]])];

      // would also be nice to have a root query for available strategies
      // that could be used by the UI, and whether they are configured.
    }
  }, {
    key: 'extendWith',
    value: function extendWith(obj) {
      _lodash2.default.extend(this, obj);
    }

    /////////////////////////////
    // require() and resolve() //
    /////////////////////////////

  }, {
    key: 'resolve',
    value: function resolve(module) {
      try {
        return require.resolve(module);
      } catch (err) {
        if (err.message === 'Cannot find module \'' + module + '\'') {

          /*
           * node resolves symlinks so that packages from npm link won't find
           * their own modules.  https://github.com/npm/npm/issues/5875
           * This is a small hack to make dev work easier.
           */
          if (module.charAt(0) !== '.') {
            var relative = _path.join(__dirname, '..', '..', module.replace(/apollo-passport-/, ''));
            try {
              return require.resolve(relative);
            } catch (err) {
              if (err.message !== 'Cannot find module \'' + relative + '\'')
                /* istanbul ignore next */
                throw err;
              return null;
            }
          }

          return null;
        }
        throw err;
      }
    }
  }, {
    key: 'require',
    value: function (_require) {
      function require(_x, _x2, _x3, _x4) {
        return _require.apply(this, arguments);
      }

      require.toString = function () {
        return _require.toString();
      };

      return require;
    }(function (strategy, module, namespace, optional) {
      // No static analysis, but that's ok for server side
      var fromPackage = 'apollo-passport-' + strategy + '/lib/' + module;

      var resolved = this.resolve(fromPackage);
      if (!resolved) resolved = this.resolve('./strategies/' + strategy + '/' + module);
      if (!resolved && namespace) resolved = this.resolve('apollo-passport-' + namespace + '/lib/' + module);
      if (!resolved && namespace) resolved = this.resolve('./strategies/' + namespace + '/' + module);

      if (!resolved) {
        if (optional) return false;else throw new Error('Cannot find \'' + fromPackage + '\'.  ' + ('Try \'npm i --save apollo-passport-' + strategy + '\'.'));
      }

      var loaded = require(resolved);
      return loaded.__esModule ? loaded.default : loaded;
    })

    ///////////
    // Users //
    ///////////

    // accept 'emails', 'services' fields
    // return userId

  }, {
    key: 'createUser',
    value: function createUser(user) {
      var userId;
      return regeneratorRuntime.async(function createUser$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              _context.next = 2;
              return regeneratorRuntime.awrap(this.db.createUser(user));

            case 2:
              userId = _context.sent;
              return _context.abrupt('return', userId);

            case 4:
            case 'end':
              return _context.stop();
          }
        }
      }, null, this);
    }

    //////////////////////
    // GraphQL & Apollo //
    //////////////////////

  }, {
    key: 'schema',
    value: function schema() {
      return this._schema;
    }
  }, {
    key: 'resolvers',
    value: function resolvers() {
      return this._bindRootQueriesAndMutations(this._resolvers);
    }

    /*
     * Wraps an apolloOptions object in a function that will check for a JWT
     * in an Authorization header, and if it passes verification, add to decoded
     * value to a 'auth' in the context.  If an error occured, adds to authError,
     * which is useful for debugging but can be safely ignored.
     */

  }, {
    key: 'wrapOptions',
    value: function wrapOptions(options) {
      var self = this;

      return function _callee(req) {
        var token, decoded;
        return regeneratorRuntime.async(function _callee$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                if (req.headers.authorization) {
                  _context2.next = 2;
                  break;
                }

                return _context2.abrupt('return', options);

              case 2:
                token = req.headers.authorization.substr(0, 7) === 'Bearer ' && req.headers.authorization.substr(7);

                if (token) {
                  _context2.next = 5;
                  break;
                }

                return _context2.abrupt('return', options);

              case 5:
                decoded = void 0;
                _context2.prev = 6;

                decoded = _jsonwebtoken2.default.verify(token, self.jwtSecret);
                _context2.next = 13;
                break;

              case 10:
                _context2.prev = 10;
                _context2.t0 = _context2['catch'](6);
                return _context2.abrupt('return', _extends({}, options, {
                  context: { authError: _context2.t0 }
                }));

              case 13:
                return _context2.abrupt('return', _extends({}, options, {
                  context: { auth: decoded }
                }));

              case 14:
              case 'end':
                return _context2.stop();
            }
          }
        }, null, this, [[6, 10]]);
      };
    }

    /*
     * Given a resolver object, it returns a new object, where every
     * function that is a sub-key of the RootMutation key will be bound
     * to the ApolloPassport instance (i.e. accessible via `this`).
     */

  }, {
    key: '_bindRootQueriesAndMutations',
    value: function _bindRootQueriesAndMutations(obj) {
      var out = {};
      for (var key in obj) {
        if (key === 'RootMutation' || key === 'RootQuery') {
          out[key] = {};
          for (var key2 in obj[key]) {
            out[key][key2] = obj[key][key2].bind(this);
          }
        } else {
          out[key] = obj[key];
        }
      }return out;
    }

    /////////////////
    // Middlewares //
    /////////////////

  }, {
    key: 'popupScript',
    value: function popupScript(data) {
      var json = JSON.stringify(data);

      // Indentation for maintainence only
      // Final string should be as small as possible to send over the wire
      return '' + '<html>' + '<head>' + '<script type="text/javascript">' + ('window.opener.postMessage(\'apolloPassport ' + json + '\', window.location.origin);') + 'window.close();' + '</script>' + '</head>' + '</html>';
    }
  }, {
    key: 'expressMiddleware',
    value: function expressMiddleware() {
      var self = this;

      return function ApolloPassportExpressMiddleware(req, res /*, next */) {
        var optParts = req.url.split('?');
        var pathParts = optParts[0].split('/');
        var strategy = pathParts[1];
        var action = pathParts[2];

        self.apAuthenticate(strategy, req.query).then(function (data) {
          res.setHeader('content-type', 'text/html');
          res.end(self.popupScript(data), 'utf8');
        }).catch(function (error) {
          console.error(error);
          res.status(500, 'Internal server error');
          res.end();
        });
      };
    }
  }]);

  return ApolloPassport;
}();

exports.defaultMapUserToJWTProps = defaultMapUserToJWTProps;
exports.defaultCreateTokenFromUser = defaultCreateTokenFromUser;
exports.default = ApolloPassport;