'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = oauth2verify;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/* Sample data

accessToken: "EAAFvTvnWMmIBAOjoduxXVU1N8yOSnPxbKlWAeZACieV5tjXJ5s4XnvHlDyc7IvnN30ZC6EV8pv4B9SY6ONE7IulGtzI4XeMbqYAorO6oV4ktwJi8DHcUoTccSkaoo89lwvZBtLOXydx2myOeAsHBKBJ6vlPaioZD"
refreshToken: "undefined"
profile: { id: '123456',
  username: undefined,
  displayName: 'John Sheppard',
  name:
   { familyName: undefined,
     givenName: undefined,
     middleName: undefined },
  gender: undefined,
  profileUrl: undefined,
  emails: [ { value: 'sheppard@atlantis.net' } ],
  provider: 'facebook',
  _raw: '{"id":"123456","name":"John Sheppard","email":"sheppard\\u0040atlantis.net"}',
  _json:
   { id: '123456',
     name: 'Gadi Cohen',
     email: 'dragon@wastelands.net' }
}
*/

function oauth2verify(strategy, accessToken, refreshToken, profile, cb) {
  var _this = this;

  // console.log('profile', profile);

  var provider = profile.provider;
  if (strategy !== provider) console.warn('[harmless warn for dev] oauth2verify, ' + ('strategy "' + strategy + '" !== provider "' + provider + '"'));

  // This is not really a "verification" but we want to store it in the database
  profile.accessToken = accessToken;
  if (refreshToken) profile.refreshToken = refreshToken;

  // support multiple email addresses?  needs DB support.  do any providers do this?
  var email = profile.emails && profile.emails[0] && profile.emails[0].value;

  // TODO double check if DB driver if email is null that all works as expected
  this.db.fetchUserByServiceIdOrEmail(provider, profile.id, email).then(function (user) {

    if (!user) {
      user = { emails: [], services: _defineProperty({}, provider, profile) };
      if (email) user.emails.push({ address: email });

      return _this.createUser(user).then(function (userId) {
        user.id = userId;
        return cb(null, user);
      }).catch(cb);
    }

    // if we're out of date, update in the background (i.e. no await)
    if (!user.services[provider] || JSON.stringify(user.services[provider]) !== JSON.stringify(profile)) {

      _this.db.assertUserServiceData(user.id, provider, profile);
      user.services[provider] = profile;
    }
    if (!user.emails) user.emails = [];
    if (email && !_lodash2.default.find(user.emails, { address: email })) {
      _this.db.assertUserEmailData(user.id, email);
      user.emails.push({ address: email });
    }

    cb(null, user);
  }).catch(cb);
}