'use strict';

var _chai = require('chai');

var _chai2 = _interopRequireDefault(_chai);

var _graphqlMerge = require('./graphql-merge');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var should = _chai2.default.should();

describe('graphql-merge', function () {

  describe('mergeResolvers', function () {

    it('merges all 1st-level keys, and 2nd-level RootQueries & RootMutations', function () {

      var a = { a: 1, RootMutation: { c: 1 }, RootQuery: { e: 1 } };
      var b = { b: 1, RootMutation: { d: 1 }, RootQuery: { f: 1 } };

      (0, _graphqlMerge.mergeResolvers)(a, b).should.deep.equal({
        a: 1, b: 1,
        RootMutation: { c: 1, d: 1 },
        RootQuery: { e: 1, f: 1 }
      });
    });
  });

  describe('mergeSchemas', function () {

    it('merges schemas', function () {

      var a = '\ntype Something {\n  error: String,\n  token: String,\n  userId: String\n}\n\ntype RootMutations {\n  someMutation (email: String!, password: String!): Something\n}\n\ntype RootQueries {\n  someQuery (email: String!, password: String!): Something\n}\n\nschema {\n  query: RootQueries\n  mutation: RootMutations\n}\n';

      var b = '\ntype SomethingElse {\n  id: String\n}\n\ntype RootMutations {\n  someOtherMutation (id: String): SomethingElse\n}\n\ntype RootQueries {\n  someOtherQuery (id: String): SomethingElse\n}\n\nschema {\n  query: RootQueries\n  mutation: RootMutations\n}\n';

      var merged = (0, _graphqlMerge.mergeSchemas)([a, b]);
      merged.should.equal('\n\ntype Something {\n  error: String,\n  token: String,\n  userId: String\n}\n\n\ntype SomethingElse {\n  id: String\n}\n\n\ntype RootQuery {\n  someQuery (email: String!, password: String!): Something\n  someOtherQuery (id: String): SomethingElse\n\n}\ntype RootMutation {\n  someMutation (email: String!, password: String!): Something\n  someOtherMutation (id: String): SomethingElse\n\n}\nschema {\n  query: RootQuery,\n  mutation: RootMutation\n}\n  ');
    });
  });
});